{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DerivingStrategies #-}
{-# LANGUAGE DerivingVia #-}
{-# LANGUAGE TypeApplications #-}

module ElmskellTypes (generateElmskellTypes) where

import Data.Aeson (ToJSON (..), FromJSON (..))
import Elm
import GHC.Generics

data Font = Font
  { fontFontSize :: Float
  } deriving (Generic)
    deriving (Elm, ToJSON, FromJSON) via ElmStreet Font

data Cookies = Cookies
  { cookiesFont :: Font
  , cookiesCookiesKept :: CookiesKept
  , cookiesTheme :: Theme
  , cookiesPrompt :: Prompt
  } deriving (Generic)
    deriving (Elm, ToJSON, FromJSON) via ElmStreet Cookies

data Theme
  = Pit
  | Dim
  | Sky
  | Sun
  deriving (Generic)
  deriving (Elm, ToJSON, FromJSON) via ElmStreet Theme

data CookiesKept = CookiesKept
  { cookiesKeptKeepTheme :: Bool
  , cookiesKeptKeepFont :: Bool
  , cookiesKeptKeepPrompt :: Bool
  } deriving (Generic)
    deriving (Elm, ToJSON, FromJSON) via ElmStreet CookiesKept

data Prompt = Prompt
  { promptPrompt :: String
  } deriving (Generic)
    deriving (Elm, ToJSON, FromJSON) via ElmStreet Prompt

data CoreColor
  = Red
  | Green
  | Yellow
  | Blue
  | Magenta
  | Cyan
  | BrightRed
  | BrightGreen
  | BrightYellow
  | BrightBlue
  | BrightMagenta
  | BrightCyan
  deriving (Generic)
  deriving (Elm, ToJSON, FromJSON) via ElmStreet CoreColor

data ThemeColor
  = Background
  | Foreground
  | Cursor
  | Black
  | White
  | BrightBlack
  | BrightWhite
  deriving (Generic)
  deriving (Elm, ToJSON, FromJSON) via ElmStreet ThemeColor

data Command
  = HelpCommand
  | ClearCommand
  | ColorsCommand
  | CookiesCommand
  | FontCommand
  | HelloCommand
  | PromptCommand
  | ThemeCommand
  | TodoCommand
  deriving (Generic)
  deriving (Elm, ToJSON, FromJSON) via ElmStreet Command

data Input = Input
  { inputCommand :: Command
  , inputArgs :: [String]
  }
  deriving (Generic)
  deriving (Elm, ToJSON, FromJSON) via ElmStreet Input

type Types =
  '[Font
  , Cookies
  , Theme
  , CookiesKept
  , Prompt
  , CoreColor
  , ThemeColor
  , Command
  , Input
  ]

generateElmskellTypes :: IO ()
generateElmskellTypes = generateElm @Types $ defaultSettings "frontend/src" ["ElmskellTypes", "Generated"]
