module ElmskellTypes.Generated.Types exposing (..)

import Time exposing (Posix)
import Json.Decode exposing (Value)


type alias Font =
    { fontSize : Float
    }

type alias Cookies =
    { font : Font
    , cookiesKept : CookiesKept
    , theme : Theme
    , prompt : Prompt
    }

type Theme
    = Pit
    | Dim
    | Sky
    | Sun

showTheme : Theme -> String
showTheme x = case x of
    Pit -> "Pit"
    Dim -> "Dim"
    Sky -> "Sky"
    Sun -> "Sun"

readTheme : String -> Maybe Theme
readTheme x = case x of
    "Pit" -> Just Pit
    "Dim" -> Just Dim
    "Sky" -> Just Sky
    "Sun" -> Just Sun
    _ -> Nothing

universeTheme : List Theme
universeTheme = [Pit, Dim, Sky, Sun]

type alias CookiesKept =
    { keepTheme : Bool
    , keepFont : Bool
    , keepPrompt : Bool
    }

type alias Prompt =
    { prompt : String
    }

type CoreColor
    = Red
    | Green
    | Yellow
    | Blue
    | Magenta
    | Cyan
    | BrightRed
    | BrightGreen
    | BrightYellow
    | BrightBlue
    | BrightMagenta
    | BrightCyan

showCoreColor : CoreColor -> String
showCoreColor x = case x of
    Red -> "Red"
    Green -> "Green"
    Yellow -> "Yellow"
    Blue -> "Blue"
    Magenta -> "Magenta"
    Cyan -> "Cyan"
    BrightRed -> "BrightRed"
    BrightGreen -> "BrightGreen"
    BrightYellow -> "BrightYellow"
    BrightBlue -> "BrightBlue"
    BrightMagenta -> "BrightMagenta"
    BrightCyan -> "BrightCyan"

readCoreColor : String -> Maybe CoreColor
readCoreColor x = case x of
    "Red" -> Just Red
    "Green" -> Just Green
    "Yellow" -> Just Yellow
    "Blue" -> Just Blue
    "Magenta" -> Just Magenta
    "Cyan" -> Just Cyan
    "BrightRed" -> Just BrightRed
    "BrightGreen" -> Just BrightGreen
    "BrightYellow" -> Just BrightYellow
    "BrightBlue" -> Just BrightBlue
    "BrightMagenta" -> Just BrightMagenta
    "BrightCyan" -> Just BrightCyan
    _ -> Nothing

universeCoreColor : List CoreColor
universeCoreColor = [ Red
                    , Green
                    , Yellow
                    , Blue
                    , Magenta
                    , Cyan
                    , BrightRed
                    , BrightGreen
                    , BrightYellow
                    , BrightBlue
                    , BrightMagenta
                    , BrightCyan ]

type ThemeColor
    = Background
    | Foreground
    | Cursor
    | Black
    | White
    | BrightBlack
    | BrightWhite

showThemeColor : ThemeColor -> String
showThemeColor x = case x of
    Background -> "Background"
    Foreground -> "Foreground"
    Cursor -> "Cursor"
    Black -> "Black"
    White -> "White"
    BrightBlack -> "BrightBlack"
    BrightWhite -> "BrightWhite"

readThemeColor : String -> Maybe ThemeColor
readThemeColor x = case x of
    "Background" -> Just Background
    "Foreground" -> Just Foreground
    "Cursor" -> Just Cursor
    "Black" -> Just Black
    "White" -> Just White
    "BrightBlack" -> Just BrightBlack
    "BrightWhite" -> Just BrightWhite
    _ -> Nothing

universeThemeColor : List ThemeColor
universeThemeColor = [ Background
                     , Foreground
                     , Cursor
                     , Black
                     , White
                     , BrightBlack
                     , BrightWhite ]

type Command
    = HelpCommand
    | ClearCommand
    | ColorsCommand
    | CookiesCommand
    | FontCommand
    | HelloCommand
    | PromptCommand
    | ThemeCommand
    | TodoCommand

showCommand : Command -> String
showCommand x = case x of
    HelpCommand -> "HelpCommand"
    ClearCommand -> "ClearCommand"
    ColorsCommand -> "ColorsCommand"
    CookiesCommand -> "CookiesCommand"
    FontCommand -> "FontCommand"
    HelloCommand -> "HelloCommand"
    PromptCommand -> "PromptCommand"
    ThemeCommand -> "ThemeCommand"
    TodoCommand -> "TodoCommand"

readCommand : String -> Maybe Command
readCommand x = case x of
    "HelpCommand" -> Just HelpCommand
    "ClearCommand" -> Just ClearCommand
    "ColorsCommand" -> Just ColorsCommand
    "CookiesCommand" -> Just CookiesCommand
    "FontCommand" -> Just FontCommand
    "HelloCommand" -> Just HelloCommand
    "PromptCommand" -> Just PromptCommand
    "ThemeCommand" -> Just ThemeCommand
    "TodoCommand" -> Just TodoCommand
    _ -> Nothing

universeCommand : List Command
universeCommand = [ HelpCommand
                  , ClearCommand
                  , ColorsCommand
                  , CookiesCommand
                  , FontCommand
                  , HelloCommand
                  , PromptCommand
                  , ThemeCommand
                  , TodoCommand ]

type alias Input =
    { command : Command
    , args : List String
    }
